/***************************************************************************
 *   Copyright (C) 2004 by Gerd Fleischer                                  *
 *   gerdfleischer@gmx.de                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "klinpopup.h"
#include "settings.h"
#include "prefs.h"
#include "makepopup.h"

#include <qdragobject.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qdir.h>
#include <qtooltip.h>

#include <kwin.h>
#include <kdebug.h>
#include <kconfigdialog.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaudioplayer.h>

#include <kedittoolbar.h>

#include <kstdaccel.h>
#include <kstdaction.h>

KLinPopup::KLinPopup()
    : KMainWindow( 0, "KLinPopup" ),
      m_view(new KLinPopupView(this)),
      actualPopup(0),
      unreadMessages(0),
      popupFileDirectory("/var/lib/klinpopup")    
{
    setFocusPolicy(QWidget::StrongFocus);
    
    // accept dnd
    setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();
    
    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    readConfig();
    
    initSystemTray();

    if (startDocked)
        isDockedOnly = true;
    
    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
   
    updateStats();

//    statusBar()->hide();
//    m_statusbarAction->setChecked(false);
    
    initTimer();
    popupFileTimer->start(5000, true);
}

KLinPopup::~KLinPopup()
{
}

void KLinPopup::closeEvent(QCloseEvent *e)
{
    // copied from kscd
    const QObject* caller = sender();
    
    while (m_systemTray && caller)
    {
        if (caller == m_systemTray)
        {
            break;
        }
        caller = caller->parent();
    }

    if (!caller)
    {
        hide();
        e->ignore();
        isDockedOnly = true;
        return;
    }

    e->accept();
    if (e->isAccepted())
        kapp->quit();
} // closeEvent

void KLinPopup::hideEvent(QHideEvent *e)
{
    KMainWindow::hideEvent(e);
    m_systemTray->show();
    hide();
    isDockedOnly = true;
} //hideEvent

void KLinPopup::focusInEvent(QFocusEvent *e)
{
    kdDebug() << "got focus" << endl;
    if (e->gotFocus()) {
        if (!checkReadStatus()) {
            unreadMessages--;
            setRead();
	}    
	updateStats();
        if (unreadMessages == 0)
            m_systemTray->changeTrayPixmap(1);
    }	
} //focusInEvent

void KLinPopup::setupActions()
{
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());

//    KStdAction::copy(kapp, SLOT(copy()), actionCollection());
    
    m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
    m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());

    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    newPopupAction = new KAction(i18n("&New"),
                                      "mail_new", 0,
				      this, SLOT(newPopup()),
			              actionCollection(), "new_popup");
    replyPopupAction = new KAction(i18n("&Reply"),
                                        "mail_reply", 0,
					this, SLOT(replyPopup()),
					actionCollection(), "reply_popup");
    previousPopupAction = new KAction(i18n("&Previous"),
                                           "previous", 0,
					   this, SLOT(previousPopup()),
					   actionCollection(), "previous_popup");
    nextPopupAction = new KAction(i18n("&Next"),
                                       "next", 0,
				       this, SLOT(nextPopup()),
				       actionCollection(), "next_popup");
    deletePopupAction = new KAction(i18n("&Delete"),
                                         "mail_delete", 0,
					 this, SLOT(deletePopup()),
					 actionCollection(), "delete_popup");
    
    createGUI();
} //setupActions

void KLinPopup::initSystemTray()
{
    m_systemTray = new SystemTray(this, "systemTray");
    m_systemTray->show();
} //initSystemtray

void KLinPopup::initTimer()
{
    // StatusBar Timer
/*    statusBarTimer = new QTimer(this);
    connect(statusBarTimer, SIGNAL(timeout()), this, SLOT(statusBarTimerDone()));*/

    // Popupfile Timer
    popupFileTimer = new QTimer(this);
    connect(popupFileTimer, SIGNAL(timeout()), this, SLOT(popupFileTimerDone()));
} //initTimer

void KLinPopup::popupFileTimerDone()
{
    QDir dir(popupFileDirectory);
    if (! dir.exists()) {
        /// @todo handle error
        perror("popup dir not exist\n");
    }
    
    const QFileInfoList *popupFiles = dir.entryInfoList(QDir::All, QDir::Time);
    if (popupFiles) {
        QFileInfoListIterator it(*popupFiles);
	QFileInfo *popupFileInfo;
        while((popupFileInfo = it.current()) != 0) {
	    ++it;
	    if (popupFileInfo -> isFile()) {
	        QString popupFileName(popupFileInfo -> fileName());
                    
		QString popupFilePath(popupFileDirectory);
                popupFilePath.append("/");
                popupFilePath.append(popupFileName);
		    
		QFile popupFile(popupFilePath);
		
		kdDebug() << "popupFile " << popupFileName.utf8() << endl;
		
                if (popupFile.open(IO_ReadOnly)) {
                    QTextStream stream(&popupFile);
                    QString line;
                    QString sender;
                    QString text;
                    QString time;
                    int i = 0;
                        
	            while (! stream.atEnd()) {
			i++;
                        line = stream.readLine();

                        if (i == 1)  {
                            time.append(line);
			    kdDebug() << "Zeit " << time.utf8() << endl;
                            continue;
                        }
                        if (i == 2) {
                            sender.append(line);
			    kdDebug() << "Sender " << sender.utf8() << endl;
                            continue;
                        }
                        if (i > 3) {
                            text.append('\n');
                        }
			text.append(line);
                    }
			
                    popupFile.close();
		    kdDebug() << "Closed" << endl;
		    
		    // delete file
                    if (! popupFile.remove()) {
                    /// @todo handle error
                    perror("can't remove message");
		    kdDebug() << "Deleted" << endl;
		    }
		    
		    signalNewMessage(sender, time, text);
	        }
	    }
	} 
    }
    popupFileTimer -> start(5000, true);
} //popupFileTimerDone
    
void KLinPopup::signalNewMessage(QString popupSender, QString popupTime, QString messageText)
{
    kdDebug() << "Popup received" << endl;

    int tmpCount;    
    tmpCount = popupSenderMap.count() + 1;
    popupSenderMap.insert(tmpCount, popupSender);
    popupTimeMap.insert(tmpCount, popupTime);
    popupMessageTextMap.insert(tmpCount, messageText);
    readStatus.insert(tmpCount, 1);
    unreadMessages++;
    
    switch (newMessageSignaling)
    {
        case 0:    //nothing
	    //show it if it's the only one
	    if (popupSenderMap.count() == 1) {
	        actualPopup=1;
	        showPopup();
	    }
	    updateStats();
            break;
	case 1:    //play sound
	    actualPopup = popupSenderMap.count();
	    showPopup();    
	    KAudioPlayer::play(newPopupSound);
	    if (isActiveWindow()) {
	        unreadMessages--;
		setRead();
	        if (unreadMessages == 0)
                    m_systemTray->changeTrayPixmap(1);
	    } else {
	        m_systemTray->changeTrayPixmap(0);
	    }	
	    updateStats();
            break;
	case 2:    //activate Window
	    actualPopup = popupSenderMap.count();
	    showPopup();
	    show();
            KWin::forceActiveWindow(winId());
	    unreadMessages--;
	    setRead();
	    if (unreadMessages == 0)
                m_systemTray->changeTrayPixmap(1);
	    updateStats();
            break;
	case 3:    //all
	    actualPopup = popupSenderMap.count();
	    showPopup();
	    KAudioPlayer::play(newPopupSound);
	    show();
            KWin::forceActiveWindow(winId());
	    unreadMessages--;
	    setRead();
	    if (unreadMessages == 0)
                m_systemTray->changeTrayPixmap(1);
	    updateStats();
            break;
    }	      
} // signalNewMessage

void KLinPopup::createStatText()
{
    QString tmpNumber;
    
    captionText = "";
    tmpNumber.setNum(actualPopup, 10);
    captionText.append(tmpNumber);
    captionText.append("/");
    tmpNumber.setNum(popupSenderMap.count(), 10);
    captionText.append(tmpNumber);
    
    statText.setNum(unreadMessages, 10);
    statText.prepend(i18n("Message ") + captionText + " - ");
    statText.append(i18n(" Unread"));
    
}

void KLinPopup::updateStats()
{
    createStatText();
    changeCaption(captionText);
    changeStatusbar(statText);
    checkMessageMap();
    QToolTip::remove(m_systemTray);
    QToolTip::add(m_systemTray, statText);
}

void KLinPopup::checkMessageMap() {
    int popupCounter = popupSenderMap.count();

    if (popupCounter != 0) {
        replyPopupAction->setEnabled(true);
	deletePopupAction->setEnabled(true);
    } else {
        replyPopupAction->setEnabled(false);
	deletePopupAction->setEnabled(false);
    }

    if (popupCounter > 1 && actualPopup > 1) {
        previousPopupAction->setEnabled(true);
    } else {
        previousPopupAction->setEnabled(false);
    }

    if (popupCounter > 1 && actualPopup < popupCounter) {
        nextPopupAction->setEnabled(true);
    } else {
        nextPopupAction->setEnabled(false);
    }
}
    
void KLinPopup::showPopup()
{
    if (!popupSenderMap.isEmpty()) {
        m_view->displayNewMessage(popupSenderMap[actualPopup],
	                          popupTimeMap[actualPopup],
				  popupMessageTextMap[actualPopup]);
    } else {
        m_view->displayNewMessage("", "", "");
    }
}
    
void KLinPopup::newPopup()
{
    makePopup *newPopupView = new makePopup(this, "New Popup", "");
    newPopupView->setCaption(i18n("New Popup"));
    newPopupView->exec();
}

void KLinPopup::replyPopup()
{
    QString sender = popupSenderMap[actualPopup];

    makePopup *replyPopupView = new makePopup(this, "Reply Popup", sender);
    replyPopupView->setCaption(i18n("Reply Popup"));
    replyPopupView->exec();
}

void KLinPopup::previousPopup()
{
    actualPopup--;
    showPopup();
    if (!checkReadStatus()) {
        unreadMessages--;
	setRead();
    }	
    if (unreadMessages == 0)
       m_systemTray->changeTrayPixmap(1);
       
    updateStats();
}

void KLinPopup::nextPopup()
{
    actualPopup++;
    showPopup();
    if (!checkReadStatus()) {
        unreadMessages--;
	setRead();
    }	
    
    if (unreadMessages == 0)
       m_systemTray->changeTrayPixmap(1);
    
    updateStats();   
}

void KLinPopup::deletePopup() {
    int countPopup = popupSenderMap.count();

    if (actualPopup == countPopup) {
        popupSenderMap.remove(actualPopup);
        popupTimeMap.remove(actualPopup);
        popupMessageTextMap.remove(actualPopup);
        actualPopup--;
    } else {
        for (int i = actualPopup; i <= countPopup - 1; i++) {
            popupSenderMap.insert(i, popupSenderMap[i + 1]);
            popupSenderMap.remove(i + 1);
            popupTimeMap.insert(i, popupTimeMap[i + 1]);
            popupTimeMap.remove(i + 1);
            popupMessageTextMap.insert(i, popupMessageTextMap[i + 1]);
            popupMessageTextMap.remove(i + 1);
        }
    }

    showPopup();
    updateStats();
}

bool KLinPopup::checkReadStatus()
{
    if (readStatus[actualPopup] == 0) {
        return true;
    } else {
        return false;
    }
}

void KLinPopup::setRead()
{
    readStatus[actualPopup] = 0;
}    

void KLinPopup::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}

void KLinPopup::optionsShowStatusbar()
{
    // show/hide the statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

void KLinPopup::optionsConfigureKeys()
{
    KKeyDialog::configure(actionCollection());
}

void KLinPopup::optionsConfigureToolbars()
{
    // use the standard toolbar editor
    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void KLinPopup::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();

    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void KLinPopup::optionsPreferences()
{
	// The preference dialog is derived from prefs-base.ui which is subclassed into Prefs
	//
	// compare the names of the widgets in the .ui file 
	// to the names of the variables in the .kcfg file
        KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self(), KDialogBase::Swallow);
        dialog->addPage(new Prefs(), i18n("General"), "package_settings");
        connect(dialog, SIGNAL(settingsChanged()), this, SLOT(settingsChanged()));
        dialog->show();
}

void KLinPopup::readConfig()
{
    startDocked = Settings::startDocked();
    newMessageSignaling = Settings::toggleSignaling();
    newPopupSound = Settings::soundURL();
}

void KLinPopup::saveConfig()
{
    Settings::writeConfig();
}
  
void KLinPopup::settingsChanged()
{
        Settings::writeConfig();
	newMessageSignaling = Settings::toggleSignaling();
	newPopupSound = Settings::soundURL();
}

void KLinPopup::changeStatusbar(const QString& text)
{
    if (!statusBar()->hasItem(ID_STATUS_TEXT)) {
        statusBar()->insertItem(text, ID_STATUS_TEXT, 1, true);
    } else {
        statusBar()->changeItem(text, ID_STATUS_TEXT);
    }	
}

void KLinPopup::changeCaption(const QString& text)
{
    setCaption(text);
}

#include "klinpopup.moc"

